@@ -1,16 +1,16 @@
-use crate::auditwheel::AuditWheelMode;
-use crate::build_options::CargoOptions;
-use crate::target::detect_arch_from_python;
 use crate::BuildContext;
 use crate::BuildOptions;
 use crate::PlatformTag;
 use crate::PythonInterpreter;
 use crate::Target;
+use crate::auditwheel::AuditWheelMode;
+use crate::build_options::CargoOptions;
+use crate::compression::CompressionOptions;
+use crate::target::detect_arch_from_python;
 use anyhow::ensure;
-use anyhow::{anyhow, bail, Context, Result};
+use anyhow::{Context, Result, anyhow, bail};
 use cargo_options::heading;
 use fs_err as fs;
-use pep508_rs::{MarkerExpression, MarkerOperator, MarkerTree, MarkerValue};
 use regex::Regex;
 use std::path::Path;
 use std::path::PathBuf;
@@ -40,6 +40,18 @@ impl InstallBackend {
 
     fn version(&self, python_path: &Path) -> Result<semver::Version> {
         let mut cmd = self.make_command(python_path);
+
+        // Newer versions of uv no longer support `uv pip --version`, and instead
+        // require that we use `uv --version`. This is a workaround to get the
+        // version of the install backend for both old and new versions of uv.
+        cmd = match self {
+            InstallBackend::Pip { .. } => cmd,
+            InstallBackend::Uv { path, args } => {
+                let mut cmd = Command::new(path);
+                cmd.args(args);
+                cmd
+            }
+        };
         let output = cmd
             .arg("--version")
             .output()
@@ -51,13 +63,14 @@ impl InstallBackend {
         let stdout = str::from_utf8(&output.stdout)?;
         let re = match self {
             InstallBackend::Pip { .. } => Regex::new(r"pip ([\w\.]+).*"),
-            InstallBackend::Uv { .. } => Regex::new(r"uv-pip ([\w\.]+).*"),
+            InstallBackend::Uv { .. } => Regex::new(r"uv ([\w\.]+).*"),
         };
-        if let Some(captures) = re.expect("regex should be valid").captures(stdout) {
-            Ok(semver::Version::parse(&captures[1])
-                .with_context(|| format!("failed to parse semver from {:?}", stdout))?)
-        } else {
-            bail!("failed to parse version from {:?}", stdout);
+        match re.expect("regex should be valid").captures(stdout) {
+            Some(captures) => Ok(semver::Version::parse(&captures[1])
+                .with_context(|| format!("failed to parse semver from {stdout:?}"))?),
+            _ => {
+                bail!("failed to parse version from {:?}", stdout);
+            }
         }
     }
 
@@ -125,24 +138,6 @@ fn find_uv_bin() -> Result<(PathBuf, Vec<&'static str>)> {
     }
 }
 
-fn check_pip_exists(python_path: &Path, pip_path: Option<&PathBuf>) -> Result<()> {
-    let output = if let Some(pip_path) = pip_path {
-        Command::new(pip_path).args(["--version"]).output()?
-    } else {
-        Command::new(python_path)
-            .args(["-m", "pip", "--version"])
-            .output()?
-    };
-    if output.status.success() {
-        let version_str =
-            str::from_utf8(&output.stdout).context("`pip --version` didn't return utf8 output")?;
-        debug!(version = %version_str, "Found pip");
-        Ok(())
-    } else {
-        bail!("`pip --version` failed with status: {}", output.status);
-    }
-}
-
 /// Detect the Python uv package
 fn find_uv_python(python_path: &Path) -> Result<(PathBuf, Vec<&'static str>)> {
     let output = Command::new(python_path)
@@ -162,6 +157,36 @@ fn find_uv_python(python_path: &Path) -> Result<(PathBuf, Vec<&'static str>)> {
     }
 }
 
+fn check_pip_exists(python_path: &Path, pip_path: Option<&PathBuf>) -> Result<()> {
+    let output = if let Some(pip_path) = pip_path {
+        Command::new(pip_path).args(["--version"]).output()?
+    } else {
+        Command::new(python_path)
+            .args(["-m", "pip", "--version"])
+            .output()?
+    };
+    if output.status.success() {
+        let version_str =
+            str::from_utf8(&output.stdout).context("`pip --version` didn't return utf8 output")?;
+        debug!(version = %version_str, "Found pip");
+        Ok(())
+    } else {
+        bail!("`pip --version` failed with status: {}", output.status);
+    }
+}
+
+/// Check if a virtualenv is created by uv by reading pyvenv.cfg
+fn is_uv_venv(venv_dir: &Path) -> bool {
+    let pyvenv_cfg = venv_dir.join("pyvenv.cfg");
+    if !pyvenv_cfg.exists() {
+        return false;
+    }
+    match fs::read_to_string(&pyvenv_cfg) {
+        Ok(content) => content.contains("\nuv = "),
+        Err(_) => false,
+    }
+}
+
 /// Install the crate as module in the current virtualenv
 #[derive(Debug, clap::Parser)]
 pub struct DevelopOptions {
@@ -174,7 +199,7 @@ pub struct DevelopOptions {
     )]
     pub bindings: Option<String>,
     /// Pass --release to cargo
-    #[arg(short = 'r', long, help_heading = heading::COMPILATION_OPTIONS,)]
+    #[arg(short = 'r', long, help_heading = heading::COMPILATION_OPTIONS, conflicts_with = "profile")]
     pub release: bool,
     /// Strip the library for minimum file size
     #[arg(long)]
@@ -206,50 +231,52 @@ pub struct DevelopOptions {
     /// Use `uv` to install packages instead of `pip`
     #[arg(long)]
     pub uv: bool,
+
+    /// Wheel compression options
+    #[command(flatten)]
+    pub compression: CompressionOptions,
 }
 
 #[instrument(skip_all)]
 fn install_dependencies(
     build_context: &BuildContext,
     extras: &[String],
-    interpreter: &PythonInterpreter,
+    python: &Path,
+    venv_dir: &Path,
     install_backend: &InstallBackend,
 ) -> Result<()> {
     if !build_context.metadata24.requires_dist.is_empty() {
+        let mut extra_names = Vec::with_capacity(extras.len());
+        for extra in extras {
+            extra_names.push(
+                pep508_rs::ExtraName::new(extra.clone())
+                    .with_context(|| format!("invalid extra name: {extra}"))?,
+            );
+        }
         let mut args = vec!["install".to_string()];
         args.extend(build_context.metadata24.requires_dist.iter().map(|x| {
             let mut pkg = x.clone();
-            // Remove extra marker to make it installable with pip
-            // Keep in sync with `Metadata21::merge_pyproject_toml()`!
-            for extra in extras {
-                pkg.marker = pkg.marker.and_then(|marker| -> Option<MarkerTree> {
-                    match marker.clone() {
-                        MarkerTree::Expression(MarkerExpression {
-                            l_value: MarkerValue::Extra,
-                            operator: MarkerOperator::Equal,
-                            r_value: MarkerValue::QuotedString(extra_value),
-                        }) if &extra_value == extra => None,
-                        MarkerTree::And(and) => match &*and {
-                            [existing, MarkerTree::Expression(MarkerExpression {
-                                l_value: MarkerValue::Extra,
-                                operator: MarkerOperator::Equal,
-                                r_value: MarkerValue::QuotedString(extra_value),
-                            })] if extra_value == extra => Some(existing.clone()),
-                            _ => Some(marker),
-                        },
-                        _ => Some(marker),
-                    }
-                });
-            }
+            // Remove extra marker to make it installable with pip:
+            //
+            // * ` and extra == 'EXTRA_NAME'`
+            // * `; extra == 'EXTRA_NAME'`
+            //
+            // Keep in sync with `Metadata23::merge_pyproject_toml()`
+            pkg.marker = pkg.marker.simplify_extras(&extra_names);
             pkg.to_string()
         }));
         let status = install_backend
-            .make_command(&interpreter.executable)
+            .make_command(python)
             .args(&args)
+            .env("VIRTUAL_ENV", venv_dir)
             .status()
-            .context("Failed to run pip install")?;
+            .with_context(|| format!("Failed to run {} install", install_backend.name()))?;
         if !status.success() {
-            bail!(r#"pip install finished with "{}""#, status)
+            bail!(
+                r#"{} install finished with "{}""#,
+                install_backend.name(),
+                status
+            )
         }
     }
     Ok(())
@@ -267,6 +294,7 @@ fn install_wheel(
     let output = cmd
         .args(["install", "--no-deps", "--force-reinstall"])
         .arg(dunce::simplified(wheel_filename))
+        .env("VIRTUAL_ENV", venv_dir)
         .output()
         .context(format!(
             "{} install failed (ran {:?} with {:?})",
@@ -294,7 +322,7 @@ fn install_wheel(
         );
     }
     if let Err(err) = configure_as_editable(build_context, python, install_backend) {
-        eprintln!("⚠️ Warning: failed to set package as editable: {}", err);
+        eprintln!("⚠️ Warning: failed to set package as editable: {err}");
     }
     Ok(())
 }
@@ -312,7 +340,7 @@ fn configure_as_editable(
     python: &Path,
     install_backend: &InstallBackend,
 ) -> Result<()> {
-    println!("✏️  Setting installed package as editable");
+    println!("✏️ Setting installed package as editable");
     install_backend.check_supports_show_files(python)?;
     let mut cmd = install_backend.make_command(python);
     let cmd = cmd.args(["show", "--files", &build_context.metadata24.name]);
@@ -363,11 +391,19 @@ pub fn develop(develop_options: DevelopOptions, venv_dir: &Path) -> Result<()> {
         extras,
         skip_install,
         pip_path,
-        cargo_options,
+        mut cargo_options,
         uv,
+        compression,
     } = develop_options;
-    let mut target_triple = cargo_options.target.as_ref().map(|x| x.to_string());
-    let target = Target::from_target_triple(cargo_options.target)?;
+    compression.validate();
+
+    // set profile to release if specified; `--release` and `--profile` are mutually exclusive
+    if release {
+        cargo_options.profile = Some("release".to_string());
+    }
+
+    let mut target_triple = cargo_options.target.clone();
+    let target = Target::from_target_triple(cargo_options.target.as_ref())?;
     let python = target.get_venv_python(venv_dir);
 
     // check python platform and architecture
@@ -394,11 +430,11 @@ pub fn develop(develop_options: DevelopOptions, venv_dir: &Path) -> Result<()> {
             target: target_triple,
             ..cargo_options
         },
+        compression,
     };
 
     let build_context = build_options
         .into_build_context()
-        .release(release)
         .strip(strip)
         .editable(true)
         .build()?;
@@ -409,8 +445,10 @@ pub fn develop(develop_options: DevelopOptions, venv_dir: &Path) -> Result<()> {
         .as_ref()
         .is_some_and(|p| !p.warn_invalid_version_info())
     {
-        bail!("Cannot build without valid version information. \
-               You need to specify either `project.version` or `project.dynamic = [\"version\"]` in pyproject.toml.");
+        bail!(
+            "Cannot build without valid version information. \
+               You need to specify either `project.version` or `project.dynamic = [\"version\"]` in pyproject.toml."
+        );
     }
 
     let interpreter =
@@ -418,10 +456,24 @@ pub fn develop(develop_options: DevelopOptions, venv_dir: &Path) -> Result<()> {
             || anyhow!("Expected `python` to be a python interpreter inside a virtualenv ಠ_ಠ"),
         )?;
 
-    let install_backend = if uv {
-        let (uv_path, uv_args) = find_uv_python(&interpreter.executable)
-            .or_else(|_| find_uv_bin())
-            .context("Failed to find uv")?;
+    let uv_venv = is_uv_venv(venv_dir);
+    let uv_info = if uv || uv_venv {
+        match find_uv_python(&interpreter.executable).or_else(|_| find_uv_bin()) {
+            Ok(uv_info) => Some(Ok(uv_info)),
+            Err(e) => {
+                if uv {
+                    Some(Err(e))
+                } else {
+                    // Ignore error and try pip instead if it's a uv venv but `--uv` is not specified
+                    None
+                }
+            }
+        }
+    } else {
+        None
+    };
+    let install_backend = if let Some(uv_info) = uv_info {
+        let (uv_path, uv_args) = uv_info?;
         InstallBackend::Uv {
             path: uv_path,
             args: uv_args,
@@ -434,7 +486,9 @@ pub fn develop(develop_options: DevelopOptions, venv_dir: &Path) -> Result<()> {
         }
     };
 
-    install_dependencies(&build_context, &extras, &interpreter, &install_backend)?;
+    if !skip_install {
+        install_dependencies(&build_context, &extras, &python, venv_dir, &install_backend)?;
+    }
 
     let wheels = build_context.build_wheels()?;
     if !skip_install {
@@ -457,7 +511,7 @@ pub fn develop(develop_options: DevelopOptions, venv_dir: &Path) -> Result<()> {
 }
 
 #[cfg(test)]
-mod test {
+mod tests {
     use std::path::PathBuf;
 
     use super::parse_direct_url_path;
@@ -480,7 +534,9 @@ Files:
   my_project-0.1.0+abc123de.dist-info/entry_points.txt
   my_project.pth
 ";
-        let expected_path = PathBuf::from("/foo bar/venv/lib/pythonABC/site-packages/my_project-0.1.0+abc123de.dist-info/direct_url.json");
+        let expected_path = PathBuf::from(
+            "/foo bar/venv/lib/pythonABC/site-packages/my_project-0.1.0+abc123de.dist-info/direct_url.json",
+        );
         assert_eq!(
             parse_direct_url_path(example_with_direct_url).unwrap(),
             Some(expected_path)
@@ -524,7 +580,9 @@ Files:\r
   my_project.pth\r
 ";
 
-        let expected_path = PathBuf::from("C:\\foo bar\\venv\\Lib\\site-packages\\my_project-0.1.0+abc123de.dist-info\\direct_url.json");
+        let expected_path = PathBuf::from(
+            "C:\\foo bar\\venv\\Lib\\site-packages\\my_project-0.1.0+abc123de.dist-info\\direct_url.json",
+        );
         assert_eq!(
             parse_direct_url_path(example_with_direct_url_windows).unwrap(),
             Some(expected_path)
